import { Client } from '../client';
import type { Models } from '../models';
import { BillingPlan } from '../enums/billing-plan';
export declare class Organizations {
    client: Client;
    constructor(client: Client);
    /**
     * Get a list of all the teams in which the current user is a member. You can use the parameters to filter your results.
     *
     * @param {string[]} queries
     * @param {string} search
     * @throws {AppwriteException}
     * @returns {Promise<Models.OrganizationList<Preferences>>}
     */
    list<Preferences extends Models.Preferences>(queries?: string[], search?: string): Promise<Models.OrganizationList<Preferences>>;
    /**
     * Create a new organization.

     *
     * @param {string} organizationId
     * @param {string} name
     * @param {BillingPlan} billingPlan
     * @param {string} paymentMethodId
     * @param {string} billingAddressId
     * @param {string[]} invites
     * @param {string} couponId
     * @param {string} taxId
     * @param {number} budget
     * @throws {AppwriteException}
     * @returns {Promise<Models.Organization<Preferences>>}
     */
    create<Preferences extends Models.Preferences>(organizationId: string, name: string, billingPlan: BillingPlan, paymentMethodId?: string, billingAddressId?: string, invites?: string[], couponId?: string, taxId?: string, budget?: number): Promise<Models.Organization<Preferences>>;
    /**
     * Delete an organization.
     *
     * @param {string} organizationId
     * @throws {AppwriteException}
     * @returns {Promise<{}>}
     */
    delete(organizationId: string): Promise<{}>;
    /**
     * Get a list of all aggregations for an organization.
     *
     * @param {string} organizationId
     * @param {string[]} queries
     * @throws {AppwriteException}
     * @returns {Promise<Models.AggregationTeamList>}
     */
    listAggregations(organizationId: string, queries?: string[]): Promise<Models.AggregationTeamList>;
    /**
     * Get a specific aggregation using it&#039;s aggregation ID.
     *
     * @param {string} organizationId
     * @param {string} aggregationId
     * @throws {AppwriteException}
     * @returns {Promise<Models.AggregationTeam>}
     */
    getAggregation(organizationId: string, aggregationId: string): Promise<Models.AggregationTeam>;
    /**
     * Set a billing address for an organization.
     *
     * @param {string} organizationId
     * @param {string} billingAddressId
     * @throws {AppwriteException}
     * @returns {Promise<Models.Organization<Preferences>>}
     */
    setBillingAddress<Preferences extends Models.Preferences>(organizationId: string, billingAddressId: string): Promise<Models.Organization<Preferences>>;
    /**
     * Delete a team&#039;s billing address.
     *
     * @param {string} organizationId
     * @throws {AppwriteException}
     * @returns {Promise<{}>}
     */
    deleteBillingAddress(organizationId: string): Promise<{}>;
    /**
     * Get a billing address using it&#039;s ID.
     *
     * @param {string} organizationId
     * @param {string} billingAddressId
     * @throws {AppwriteException}
     * @returns {Promise<Models.BillingAddress>}
     */
    getBillingAddress(organizationId: string, billingAddressId: string): Promise<Models.BillingAddress>;
    /**
     * Set the current billing email for the organization.
     *
     * @param {string} organizationId
     * @param {string} billingEmail
     * @throws {AppwriteException}
     * @returns {Promise<Models.Organization<Preferences>>}
     */
    setBillingEmail<Preferences extends Models.Preferences>(organizationId: string, billingEmail: string): Promise<Models.Organization<Preferences>>;
    /**
     * Update the budget limit for an organization.
     *
     * @param {string} organizationId
     * @param {number} budget
     * @param {number[]} alerts
     * @throws {AppwriteException}
     * @returns {Promise<Models.Organization<Preferences>>}
     */
    updateBudget<Preferences extends Models.Preferences>(organizationId: string, budget?: number, alerts?: number[]): Promise<Models.Organization<Preferences>>;
    /**
     * List all credits for an organization.

     *
     * @param {string} organizationId
     * @param {string[]} queries
     * @throws {AppwriteException}
     * @returns {Promise<Models.CreditList>}
     */
    listCredits(organizationId: string, queries?: string[]): Promise<Models.CreditList>;
    /**
     * Add credit to an organization using a coupon.
     *
     * @param {string} organizationId
     * @param {string} couponId
     * @throws {AppwriteException}
     * @returns {Promise<Models.Credit>}
     */
    addCredit(organizationId: string, couponId: string): Promise<Models.Credit>;
    /**
     * Get credit details.
     *
     * @param {string} organizationId
     * @param {string} creditId
     * @throws {AppwriteException}
     * @returns {Promise<Models.Credit>}
     */
    getCredit(organizationId: string, creditId: string): Promise<Models.Credit>;
    /**
     * List all invoices for an organization.
     *
     * @param {string} organizationId
     * @param {string[]} queries
     * @throws {AppwriteException}
     * @returns {Promise<Models.InvoiceList>}
     */
    listInvoices(organizationId: string, queries?: string[]): Promise<Models.InvoiceList>;
    /**
     * Get an invoice by its unique ID.
     *
     * @param {string} organizationId
     * @param {string} invoiceId
     * @throws {AppwriteException}
     * @returns {Promise<Models.Invoice>}
     */
    getInvoice(organizationId: string, invoiceId: string): Promise<Models.Invoice>;
    /**
     * Download invoice in PDF
     *
     * @param {string} organizationId
     * @param {string} invoiceId
     * @throws {AppwriteException}
     * @returns {Promise<Models.PaymentMethod>}
     */
    getInvoiceDownload(organizationId: string, invoiceId: string): Promise<Models.PaymentMethod>;
    /**
     * Initiate payment for failed invoice to pay live from console
     *
     * @param {string} organizationId
     * @param {string} invoiceId
     * @param {string} paymentMethodId
     * @throws {AppwriteException}
     * @returns {Promise<Models.Invoice>}
     */
    createInvoicePayment(organizationId: string, invoiceId: string, paymentMethodId: string): Promise<Models.Invoice>;
    /**
     * Validates the payment linked with the invoice and updates the invoice status if the payment status is changed.
     *
     * @param {string} organizationId
     * @param {string} invoiceId
     * @throws {AppwriteException}
     * @returns {Promise<Models.Invoice>}
     */
    validateInvoice(organizationId: string, invoiceId: string): Promise<Models.Invoice>;
    /**
     * View invoice in PDF
     *
     * @param {string} organizationId
     * @param {string} invoiceId
     * @throws {AppwriteException}
     * @returns {Promise<Models.PaymentMethod>}
     */
    getInvoiceView(organizationId: string, invoiceId: string): Promise<Models.PaymentMethod>;
    /**
     * Set a organization&#039;s default payment method.
     *
     * @param {string} organizationId
     * @param {string} paymentMethodId
     * @throws {AppwriteException}
     * @returns {Promise<Models.Organization<Preferences>>}
     */
    setDefaultPaymentMethod<Preferences extends Models.Preferences>(organizationId: string, paymentMethodId: string): Promise<Models.Organization<Preferences>>;
    /**
     * Delete the default payment method for an organization.
     *
     * @param {string} organizationId
     * @throws {AppwriteException}
     * @returns {Promise<Models.Organization<Preferences>>}
     */
    deleteDefaultPaymentMethod<Preferences extends Models.Preferences>(organizationId: string): Promise<Models.Organization<Preferences>>;
    /**
     * Set an organization&#039;s backup payment method.

     *
     * @param {string} organizationId
     * @param {string} paymentMethodId
     * @throws {AppwriteException}
     * @returns {Promise<Models.Organization<Preferences>>}
     */
    setBackupPaymentMethod<Preferences extends Models.Preferences>(organizationId: string, paymentMethodId: string): Promise<Models.Organization<Preferences>>;
    /**
     * Delete a backup payment method for an organization.
     *
     * @param {string} organizationId
     * @throws {AppwriteException}
     * @returns {Promise<Models.Organization<Preferences>>}
     */
    deleteBackupPaymentMethod<Preferences extends Models.Preferences>(organizationId: string): Promise<Models.Organization<Preferences>>;
    /**
     * Get an organization&#039;s payment method using it&#039;s payment method ID.
     *
     * @param {string} organizationId
     * @param {string} paymentMethodId
     * @throws {AppwriteException}
     * @returns {Promise<Models.PaymentMethod>}
     */
    getPaymentMethod(organizationId: string, paymentMethodId: string): Promise<Models.PaymentMethod>;
    /**
     * Get the details of the current billing plan for an organization.
     *
     * @param {string} organizationId
     * @throws {AppwriteException}
     * @returns {Promise<Models.BillingPlan>}
     */
    getPlan(organizationId: string): Promise<Models.BillingPlan>;
    /**
     * Update the billing plan for an organization.
     *
     * @param {string} organizationId
     * @param {BillingPlan} billingPlan
     * @param {string} paymentMethodId
     * @param {string} billingAddressId
     * @param {string[]} invites
     * @param {string} couponId
     * @param {string} taxId
     * @param {number} budget
     * @throws {AppwriteException}
     * @returns {Promise<Models.Organization<Preferences>>}
     */
    updatePlan<Preferences extends Models.Preferences>(organizationId: string, billingPlan: BillingPlan, paymentMethodId?: string, billingAddressId?: string, invites?: string[], couponId?: string, taxId?: string, budget?: number): Promise<Models.Organization<Preferences>>;
    /**
     * Get Scopes
     *
     * @param {string} organizationId
     * @throws {AppwriteException}
     * @returns {Promise<Models.Roles>}
     */
    getScopes(organizationId: string): Promise<Models.Roles>;
    /**
     * Set an organization&#039;s billing tax ID.
     *
     * @param {string} organizationId
     * @param {string} taxId
     * @throws {AppwriteException}
     * @returns {Promise<Models.Organization<Preferences>>}
     */
    setBillingTaxId<Preferences extends Models.Preferences>(organizationId: string, taxId: string): Promise<Models.Organization<Preferences>>;
}
